#include <cassert>
#include <stdexcept>

#include "KalahEngine.h"
#include "Player.h"
#include "Board.h"
    
IEngine::MoveResult KalahEngine::makeMove (const Player * player, Board * board, int house)
{
    // TODO: Use an board iterator instead.
    // We should not have to guess the relation between holes in this class
    Board::Half playersHalf = player->half();
    assert (playersHalf == Board::SOUTH || playersHalf == Board::NORTH);

    // Check validity of move
    int hole;
    try
    {
        hole = board->house2hole(house);        
    }
    catch (...) // insert correct exception: invalid_argument
    {
        return INVALID_MOVE;   
    }

    if (board->getHalf(hole) != playersHalf)
        return INVALID_MOVE;

    int seeds = (*board)[hole];
    
    if (seeds == 0)
        return INVALID_MOVE;

    // Move seeds
    (*board)[hole] = 0;
    int opponentStoreHole = board->getStoreHole((playersHalf == Board::SOUTH) ? Board::NORTH : Board::SOUTH);
    while (seeds)
    {
        hole = ( hole == static_cast<int>( board->size())-1 ) ? 0 : hole + 1;
        if (hole != opponentStoreHole)
        {
            ++(*board)[hole];
            --seeds;
        }
    }


    // If the last sown seed lands in an empty house owned by the player, and
    // the opposite house contains seeds, both the last seed and the opposite
    // seeds are captured and placed into the player's store.
    int storeHole = board->getStoreHole(playersHalf);
    int oppositeHole = board->getOppositeHole(hole);

    if ( (*board)[hole] == 1 &&
         board->getHalf(hole) == playersHalf &&
         hole != storeHole &&
         (*board)[oppositeHole] > 0 )
    {
        int oppositeHole = board->getOppositeHole(hole);

        (*board)[storeHole] += (*board)[hole] + (*board)[oppositeHole];
        (*board)[hole] = 0;
        (*board)[oppositeHole] = 0;
    }
    
    // Check if game is finished
    if ( board->isHalfEmpty( playersHalf ) )
    {
        // Move remaining seeds on opponents half into his/her store
        int i = storeHole + 1;
        while ( i != opponentStoreHole )
        {
            (*board)[opponentStoreHole] += (*board)[i];
            (*board)[i] = 0;
            i = ( i == static_cast<int>( board->size())-1 ) ? 0 : i + 1;            
        }
        return VALID_MOVE_GAME_FINISHED;
    }

    // If last seed is sown in own store the player gets an extra turn
    return hole == storeHole ? VALID_MOVE_EXTRA_TURN : VALID_MOVE_CHANGE_TURN;
}


/**
 * Must only be called when a game is finished. If not, the return value is undefined.
 *
 * Returns the winning player, or 0 if the game ends in a draw.
 */
Player * KalahEngine::getWinner (Player * p1, Player * p2, const Board * b) const
{
    int point1 = b->getSeedsInStore( 1 );
    int point2 = b->getSeedsInStore( 2 );
    Player * winner = 0;

    if (point1 > point2)
        winner = p1;
    else if (point2 > point1)
        winner = p2;            
    
    return winner;
}
