#include "MainWindow.h"
#include "Session.h"

#include <QString>
#include <string>

MainWindow::MainWindow ( Session * session, QMainWindow * parent )
    :  QMainWindow( parent ), session_(session)
{
    setupUi( this );
    
    houses_.push_back( house1 );
    houses_.push_back( house2 );
    houses_.push_back( house3 );
    houses_.push_back( house4 );
    houses_.push_back( house5 );
    houses_.push_back( house6 );
    houses_.push_back( house7 );
    houses_.push_back( house8 );
    houses_.push_back( house9 );
    houses_.push_back( house10 );
    houses_.push_back( house11 );
    houses_.push_back( house12 );

    for (size_t i = 0 ; i < houses_.size(); ++i )
    {
        houses_[i]->setId( i + 1);
        QObject::connect( houses_[i], SIGNAL( clicked(int) ),
            this, SLOT( houseClicked(int) ) );
    }

    storages_.push_back( storage1 );
    storages_.push_back( storage2 );

}


void MainWindow::init()
{
    if (session_->game())
    {
        player1->setText( session_->game()->getPlayer1Name().c_str() );
        player2->setText( session_->game()->getPlayer2Name().c_str() );

        update();
    }
}

void MainWindow::update()
{
    for (size_t i = 0 ; i < houses_.size(); ++i )
        houses_[i]->setNum( session_->game()->getSeedsInHouse(i+1) );

    for (size_t i = 0 ; i < storages_.size(); ++i )
        storages_[i]->setNum( session_->game()->getSeedsInStorage(i+1) );

    if (session_->game()->isFinished())
    {
        QString winner = session_->game()->getWinner().c_str();
        if (winner != "")
            status->setText(winner + " wins!");
        else
            status->setText("It's a draw!");
    }
    else
    {
        // Change font on players name to show current player
        QFont font1(player1->font());
        QFont font2(player2->font());
        QString activePlayer = session_->game()->getActivePlayerName().c_str();
        // todo: this approach does not work when players have same name.
        if (player1->text() == activePlayer)
        {
            font1.setUnderline(true);
            font1.setBold(true);
            font2.setUnderline(false);
            font2.setBold(false);
        }
        else
        {
            font1.setUnderline(false);
            font1.setBold(false);
            font2.setUnderline(true);
            font2.setBold(true);
        }
        player1->setFont(font1);
        player2->setFont(font2);
        status->setText("Waiting for " + activePlayer + " To make a move...");
    }

    QMainWindow::update();
}


void MainWindow::houseClicked (int id)
{
    if ( session_->game()->makeMove(id) )
        update();
}
